#include "FtpClient.h"
#include <iostream>

FtpClient::FtpClient(){
	setupUi(this);

	cd = new ConnectDialog(this);
	QObject::connect(action_Connect, SIGNAL(activated()), cd, SLOT(show()));
	QObject::connect(cd, SIGNAL(accepted()), this, SLOT(connect()));

	localModel = new QDirModel(this);
	localModel->setFilter(QDir::AllEntries);
	localView->setModel(localModel);
	localView->setRootIndex(localModel->index(QDir::currentPath()));
	QObject::connect(localView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(changeLocalDir(const QModelIndex&)));

	ftp = new QFtp(this);
	QObject::connect(ftp, SIGNAL(stateChanged(int)), this, SLOT(getState(int)));
	QObject::connect(ftp, SIGNAL(listInfo(const QUrlInfo&)), this, SLOT(getListItem(const QUrlInfo&)));
	QObject::connect(ftp, SIGNAL(commandFinished(int, bool)), this, SLOT(commandFinished(int, bool)));

	remoteModel = new QStandardItemModel(this);
	remoteView->setModel(remoteModel);
	QObject::connect(remoteView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(changeRemoteDir(const QModelIndex&)));

	QObject::connect(actionE_xit, SIGNAL(activated()), this, SLOT(close()));
	QObject::connect(downloadButton, SIGNAL(clicked()), this, SLOT(download()));

	file = NULL;
}

void FtpClient::connect(){
	ftp->connectToHost(cd->address->text());	

	ftp->login(cd->login->text(), cd->password->text());

	QStandardItem* item = new QStandardItem;
	item->setText("..");
	remoteModel->appendRow(item);

	ftp->list();
}

void FtpClient::getState(int state){
	QString states[6];
	states[0] = tr("Not connected");
	states[1] = tr("Host name lookup in progress...");
	states[2] = tr("Connecting...");
	states[3] = tr("Connected");
	states[4] = tr("Logged in");
	states[5] = tr("Closing...");

	statusbar->showMessage(states[state]);
}

void FtpClient::getListItem(const QUrlInfo& i){
	QStandardItem* item = new QStandardItem;

	item->setText(i.name());

	remoteModel->appendRow(item);
}

void FtpClient::changeLocalDir(const QModelIndex& index){
	localView->setRootIndex(index);
	QDir::setCurrent(localModel->filePath(index));
}

void FtpClient::changeRemoteDir(const QModelIndex& index){
	QStandardItem* item = remoteModel->itemFromIndex(index);
	QString directory = item->text();

	item = new QStandardItem;
	item->setText("..");
	remoteModel->clear();
	remoteModel->appendRow(item);

	ftp->cd(directory);
	ftp->list();
}

void FtpClient::download(){
	if(file == NULL){
		QModelIndex index  = remoteView->currentIndex();
		QStandardItem* item = remoteModel->itemFromIndex(index);

		file = new QFile(item->text());

		if (!file->open(QIODevice::WriteOnly)) {
			QMessageBox::information(this, tr("FTP"),
			tr("Unable to save the file %1: %2.")
			.arg(item->text()).arg(file->errorString()));
			delete file;
			return;
		}

		ftp->get(item->text(), file);
	} else {
		statusbar->showMessage("Poczkeja..");
	}
}

void FtpClient::upload(){
}

void FtpClient::commandFinished(int id, bool error){
	if(file != NULL){
		file->close();
		delete file;
		file = NULL;
		localModel->refresh();
		statusbar->showMessage(tr("Data transfer finished..."));
	}
}
